@@ -14,7 +14,7 @@ display="$7"
 
 if [ ".`grep $cookiestr $cookiefile`" = . ]; then
     cat $patchfile |\
-    $patchprg --forward --directory=$applydir 2>&1 |\
+    $patchprg --forward --directory $applydir 2>&1 |\
     tee config.log |\
     egrep '^.Index:' | sed -e "s/.*Index: /$prefix patching: [FILE] /" |\
     eval $display
