@@ -62,7 +62,8 @@
 **  Implementation of a Generic Hook Interface for Apache
 **  Written by Ralf S. Engelschall <rse@engelschall.com> 
 **
-**  See ap_hook.h for documentation.
+**  See POD document at end of ap_hook.h for description.
+**  View it with the command ``pod2man ap_hook.h | nroff -man | more''
 **
 **  Attention: This source file is a little bit tricky.
 **             It's a combination of a C source and an embedded Perl script
@@ -208,7 +209,7 @@ API_EXPORT(int) ap_hook_configure(char *hook, ap_hook_sig sig, ap_hook_mode mode
     else {
         he->he_sig = sig;
         he->he_modeid = modeid;
-        if (modeid == AP_HOOK_MODE_DECLINE) {
+        if (modeid == AP_HOOK_MODE_DECLINE || modeid == AP_HOOK_MODE_DECLTMP) {
             if (AP_HOOK_SIG_HAS(sig, RC, char))
                 he->he_modeval.v_char = va_arg(ap, va_type(char));
             else if (AP_HOOK_SIG_HAS(sig, RC, int))
@@ -313,7 +314,7 @@ API_EXPORT(int) ap_hook_use(char *hook, ap_hook_sig sig, ap_hook_mode modeid, ..
 
     va_start(ap, modeid);
 
-    if (modeid == AP_HOOK_MODE_DECLINE) {
+    if (modeid == AP_HOOK_MODE_DECLINE || modeid == AP_HOOK_MODE_DECLTMP) {
         if (AP_HOOK_SIG_HAS(sig, RC, char))
             modeval.v_char = va_arg(ap, va_type(char));
         else if (AP_HOOK_SIG_HAS(sig, RC, int))
@@ -393,6 +394,7 @@ API_EXPORT(int) ap_hook_call(char *hook, ...)
 static int ap_hook_call_func(va_list ap, ap_hook_entry *he, ap_hook_func *hf)
 {
     void *v_rc;
+    ap_hook_value v_tmp;
     int rc;
 
     /*
@@ -473,8 +475,27 @@ static int ap_hook_call_func(va_list ap, ap_hook_entry *he, ap_hook_func *hf)
 
     rc = TRUE;
     v_rc = NULL;
-    if (!AP_HOOK_SIG_HAS(he->he_sig, RC, void))
-        v_rc = va_arg(ap, void *);
+    if (!AP_HOOK_SIG_HAS(he->he_sig, RC, void)) {
+        if (he->he_modeid == AP_HOOK_MODE_DECLTMP) {
+            /* the return variable is a temporary one */ 
+            if (AP_HOOK_SIG_HAS(he->he_sig, RC, char))
+                v_rc = &v_tmp.v_char;
+            else if (AP_HOOK_SIG_HAS(he->he_sig, RC, int))
+                v_rc = &v_tmp.v_int;
+            else if (AP_HOOK_SIG_HAS(he->he_sig, RC, long))
+                v_rc = &v_tmp.v_long;
+            else if (AP_HOOK_SIG_HAS(he->he_sig, RC, float))
+                v_rc = &v_tmp.v_float;
+            else if (AP_HOOK_SIG_HAS(he->he_sig, RC, double))
+                v_rc = &v_tmp.v_double;
+            else if (AP_HOOK_SIG_HAS(he->he_sig, RC, ptr))
+                v_rc = &v_tmp.v_ptr;
+        }
+        else {
+            /* the return variable is provided by caller */ 
+            v_rc = va_arg(ap, void *);
+        }
+    }
 
     /* ----BEGIN GENERATED SECTION-------- */
     if (he->he_sig == AP_HOOK_SIG1(void)) {
