@@ -181,7 +181,7 @@ module MODULE_VAR_EXPORT ssl_module = {
     ssl_config_server_merge,  /* merge  per-server config structures */
     ssl_config_cmds,          /* table of config file commands       */
     ssl_config_handler,       /* [#8] MIME-typed-dispatched handlers */
-    NULL,                     /* [#1] URI to filename translation    */
+    ssl_hook_Translate,       /* [#1] URI to filename translation    */
     ssl_hook_Auth,            /* [#4] validate user id from request  */
     NULL,                     /* [#5] check if the user is ok _here_ */
     ssl_hook_Access,          /* [#3] check access by host address   */
