@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  ssl_engine_init.c
@@ -164,6 +164,11 @@ void ssl_init_Module(server_rec *s, pool *p)
         ssl_log(s, SSL_LOG_INFO, "Init: %d%s restart round (already detached)",
                 mc->nInitCount-2, (mc->nInitCount-2) == 1 ? "st" : "nd");
 
+#ifdef SSL_VENDOR
+    ap_hook_use("ssl::vendor::init_module", 
+                AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, p);
+#endif
+
     /*
      *  The initialization phase inside the Apache API is totally bogus.
      *  We actually have three non-trivial problems:
@@ -305,7 +310,7 @@ void ssl_init_SSLLibrary(server_rec *s)
     CRYPTO_malloc_init();
 #endif
     SSL_load_error_strings();
-    SSLeay_add_ssl_algorithms();
+    OpenSSL_add_ssl_algorithms();
     ssl_util_thread_setup();
     return;
 }
@@ -321,6 +326,7 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
     SSL_CTX *ctx;
     STACK *skCAList;
     ssl_asn1_t *asn1;
+    unsigned char *ucp;
     char *cp;
 
     /*
@@ -342,7 +348,7 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
     /*
      *  Check for problematic re-initializations
      */
-    if (sc->px509Certificate) {
+    if (sc->px509Certificate != NULL) {
         ssl_log(s, SSL_LOG_ERROR,
                 "Init: (%s) Illegal attempt to re-initialise SSL for server "
                 "(theoretically shouldn't happen!)", cpVHostID);
@@ -454,7 +460,13 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
                 "Init: (%s) Ops, can't find server certificate?!", cpVHostID);
         ssl_die();
     }
-    sc->px509Certificate = d2i_X509(NULL, &(asn1->cpData), asn1->nData);
+    ucp = asn1->cpData;
+    if ((sc->px509Certificate = d2i_X509(NULL, &ucp, asn1->nData)) == NULL) {
+        ssl_log(s, SSL_LOG_ERROR,
+                "Init: (%s) Failed to convert server certificate "
+                "from DER to internal format?!", cpVHostID);
+        ssl_die();
+    }
 
     /*
      *  Configure server private key
@@ -466,7 +478,13 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
                 "Init: (%s) Ops, can't find server private key?!", cpVHostID);
         ssl_die();
     }
-    sc->prsaKey = d2i_RSAPrivateKey(NULL, &(asn1->cpData), asn1->nData);
+    ucp = asn1->cpData;
+    if ((sc->prsaKey = d2i_RSAPrivateKey(NULL, &ucp, asn1->nData)) == NULL) {
+        ssl_log(s, SSL_LOG_ERROR,
+                "Init: (%s) Failed to convert server private key "
+                "from DER to internal format?!", cpVHostID);
+        ssl_die();
+    }
 
     return;
 }
