@@ -132,6 +132,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
     SSLSrvConfigRec *sc;
     SSL *ssl;
     char *cp;
+    char *cpVHostID;
     int rc;
     int n;
 
@@ -158,8 +159,9 @@ void ssl_hook_NewConnection(conn_rec *conn)
      * Remember the connection information for
      * later access inside callback functions
      */
+    cpVHostID = ssl_util_vhostid(conn->pool, srvr);
     ssl_log(srvr, SSL_LOG_INFO, "Connection to child %d established (server %s)",
-            conn->child_num, ssl_util_vhostid(conn->pool, srvr));
+            conn->child_num, cpVHostID);
 
     /*
      * Seed the Pseudo Random Number Generator (PRNG)
@@ -242,7 +244,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 ssl_log(srvr, SSL_LOG_INFO,
                         "SSL handshake stopped: connection was closed");
                 SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-                while (!SSL_shutdown(ssl));
+                SSL_smart_shutdown(ssl);
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
                 ap_bsetflag(fb, B_EOF|B_EOUT, 1);
@@ -285,7 +287,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
 
                 /* third: kick away the SSL stuff */
                 SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
-                while (!SSL_shutdown(ssl));
+                SSL_smart_shutdown(ssl);
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
 
@@ -295,9 +297,9 @@ void ssl_hook_NewConnection(conn_rec *conn)
             else if (ap_ctx_get(ap_global_ctx, "ssl::handshake::timeout") == (void *)TRUE) {
                 ssl_log(srvr, SSL_LOG_ERROR, 
                         "SSL handshake timed out (client %s, server %s)",
-                        conn->remote_ip, ssl_util_vhostid(conn->pool, srvr));
+                        conn->remote_ip, cpVHostID);
                 SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-                while (!SSL_shutdown(ssl));
+                SSL_smart_shutdown(ssl);
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
                 ap_bsetflag(fb, B_EOF|B_EOUT, 1);
@@ -310,7 +312,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                         "SSL handshake interrupted by system");
                 SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-                while (!SSL_shutdown(ssl));
+                SSL_smart_shutdown(ssl);
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
                 ap_bsetflag(fb, B_EOF|B_EOUT, 1);
@@ -323,7 +325,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
                  */
                 ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR|SSL_ADD_ERRNO,
                         "SSL handshake failed (client %s, server %s)",
-                        conn->remote_ip, ssl_util_vhostid(conn->pool, srvr));
+                        conn->remote_ip, cpVHostID);
 
                 /*
                  * try to gracefully shutdown the connection:
@@ -333,7 +335,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
                  * - block the socket, so Apache cannot operate any more
                  */
                 SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-                while (!SSL_shutdown(ssl));
+                SSL_smart_shutdown(ssl);
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
                 ap_bsetflag(fb, B_EOF|B_EOUT, 1);
@@ -349,7 +351,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
             ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR,
                     "SSL client authentication failed: %s", cp);
             SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-            while (!SSL_shutdown(ssl));
+            SSL_smart_shutdown(ssl);
             SSL_free(ssl);
             ap_ctx_set(fb->ctx, "ssl", NULL);
             ap_bsetflag(fb, B_EOF|B_EOUT, 1);
@@ -381,7 +383,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
             ssl_log(srvr, SSL_LOG_ERROR,
                     "No acceptable peer certificate available");
             SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-            while (!SSL_shutdown(ssl));
+            SSL_smart_shutdown(ssl);
             SSL_free(ssl);
             ap_ctx_set(fb->ctx, "ssl", NULL);
             ap_bsetflag(fb, B_EOF|B_EOUT, 1);
@@ -427,28 +429,60 @@ void ssl_hook_TimeoutConnection(int sig)
 
 /*
  *  Close the SSL part of the socket connection
+ *  (called immediately _before_ the socket is closed)
  */
 void ssl_hook_CloseConnection(conn_rec *conn)
 {
     SSL *ssl;
 
-    /*
-     * Optionally shutdown the SSL session
-     */
     ssl = ap_ctx_get(conn->client->ctx, "ssl");
-    if (ssl != NULL) {
-        SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
-        while (!SSL_shutdown(ssl));
-        SSL_free(ssl);
-        ap_ctx_set(conn->client->ctx, "ssl", NULL);
-    }
+    if (ssl == NULL)
+        return;
 
     /*
-     * And finally log the connection close
+     * Now close the SSL layer of the connection. We've to take
+     * the TLSv1 standard into account here:
+     *
+     * | 7.2.1. Closure alerts
+     * |  
+     * | The client and the server must share knowledge that the connection is
+     * | ending in order to avoid a truncation attack. Either party may
+     * | initiate the exchange of closing messages.
+     * |  
+     * | close_notify
+     * |     This message notifies the recipient that the sender will not send
+     * |     any more messages on this connection. The session becomes
+     * |     unresumable if any connection is terminated without proper
+     * |     close_notify messages with level equal to warning.
+     * | 
+     * | Either party may initiate a close by sending a close_notify alert.
+     * | Any data received after a closure alert is ignored.
+     * | 
+     * | Each party is required to send a close_notify alert before closing
+     * | the write side of the connection. It is required that the other party
+     * | respond with a close_notify alert of its own and close down the
+     * | connection immediately, discarding any pending writes. It is not
+     * | required for the initiator of the close to wait for the responding
+     * | close_notify alert before closing the read side of the connection.
+     *
+     * This means we've to send a close notify message, but haven't to wait
+     * for the close notify of the client. Actually we cannot wait for the
+     * close notify of the client because some clients (including Netscape
+     * 4.x) don't send one, so we would hang.
      */
-    ssl_log(conn->server, SSL_LOG_INFO, "Connection to child %d closed (server %s)",
-            conn->child_num, ssl_util_vhostid(conn->pool, conn->server));
 
+    /* send close notify message */
+    SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
+    SSL_smart_shutdown(ssl);
+
+    /* deallocate the SSL connection */
+    SSL_free(ssl);
+    ap_ctx_set(conn->client->ctx, "ssl", NULL);
+
+    /* and finally log the fact that we've closed the connection */
+    ssl_log(conn->server, SSL_LOG_INFO, 
+            "Connection to child %d closed (server %s)",
+            conn->child_num, ssl_util_vhostid(conn->pool, conn->server));
     return;
 }
 
@@ -483,6 +517,26 @@ int ssl_hook_ReadReq(request_rec *r)
     return DECLINED;
 }
 
+/*
+ *  URL Translation Handler
+ */
+int ssl_hook_Translate(request_rec *r)
+{
+    if (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
+        return DECLINED;
+
+    ssl_log(r->server, SSL_LOG_INFO, 
+            "%s HTTPS request received for child %d (server %s)",
+            r->connection->keepalives <= 0 ? 
+                "Initial (No.1)" :
+                ap_psprintf(r->pool, "Subsequent (No.%d)", 
+                            r->connection->keepalives+1),
+            r->connection->child_num, 
+            ssl_util_vhostid(r->pool, r->server));
+
+    return DECLINED;
+}
+
 /*
  *  Content Handler
  */
@@ -801,10 +855,10 @@ int ssl_hook_Access(request_rec *r)
     }
 
     /*
-	 * Under old OpenSSL we had to change the X509_STORE inside the
-	 * SSL_CTX instead inside the SSL structure, so we have to reconfigure it
-	 * to the old values. This should be changed with forthcoming OpenSSL
-	 * versions when better functionality is avaiable.
+     * Under old OpenSSL we had to change the X509_STORE inside the
+     * SSL_CTX instead inside the SSL structure, so we have to reconfigure it
+     * to the old values. This should be changed with forthcoming OpenSSL
+     * versions when better functionality is avaiable.
      */
 #ifdef SSL_EXPERIMENTAL
     if (renegotiate && reconfigured_locations) {
