@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  ssl_engine_scache.c
@@ -73,7 +73,7 @@
 */
 
 /*
- *  FIXME: There is no define in SSLeay, but SSLeay uses 1024*10,
+ *  FIXME: There is no define in OpenSSL, but OpenSSL uses 1024*10,
  *  so 1024*20 should be ok.
  */
 #define MAX_SESSION_DER 1024*20
@@ -85,6 +85,12 @@ void ssl_scache_init(server_rec *s, pool *p)
     if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
         ssl_scache_dbm_init(s, p);
     ssl_scache_expire(s);
+
+#ifdef SSL_VENDOR
+    ap_hook_use("ssl::vendor::scache_init", 
+                AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, p);
+#endif
+
     return;
 }
 
@@ -108,6 +114,11 @@ void ssl_scache_store(server_rec *s, SSL_SESSION *pSession, int timeout)
     if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
         ssl_scache_dbm_store(s, &SCI);
 
+#ifdef SSL_VENDOR
+    ap_hook_use("ssl::vendor::scache_store", 
+                AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, &SCI);
+#endif
+
     return;
 }
 
@@ -129,6 +140,11 @@ SSL_SESSION *ssl_scache_retrieve(server_rec *s, UCHAR *id, int idlen)
     if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
         ssl_scache_dbm_retrieve(s, &SCI);
 
+#ifdef SSL_VENDOR
+    ap_hook_use("ssl::vendor::scache_retrieve", 
+                AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, &SCI);
+#endif
+
     /* return immediately if not found */
     if (SCI.ucaData == NULL)
         return NULL;
@@ -138,6 +154,10 @@ SSL_SESSION *ssl_scache_retrieve(server_rec *s, UCHAR *id, int idlen)
     if (SCI.tExpiresAt <= tNow) {
         if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
             ssl_scache_dbm_remove(s, &SCI);
+#ifdef SSL_VENDOR
+        ap_hook_use("ssl::vendor::scache_remove", 
+                    AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, &SCI);
+#endif
         return NULL;
     }
 
@@ -162,6 +182,11 @@ void ssl_scache_remove(server_rec *s, SSL_SESSION *pSession)
     if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
         ssl_scache_dbm_remove(s, &SCI);
 
+#ifdef SSL_VENDOR
+    ap_hook_use("ssl::vendor::scache_remove", 
+                AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, &SCI);
+#endif
+
     return;
 }
 
