@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  ssl_engine_vars.c
@@ -532,7 +532,7 @@ static char *ssl_var_lookup_ssl_cipher(pool *p, conn_rec *c, char *var)
 
 /*
  * This structure is used instead of SSL_get_cipher_bits() because
- * this SSLeay function has rounding problems, but we want the
+ * this OpenSSL function has rounding problems, but we want the
  * correct sizes.
  */
 static const struct {
@@ -540,6 +540,11 @@ static const struct {
     int nUseKeySize;
     int nAlgKeySize;
 } ssl_var_lookup_ssl_cipher_bits_rec[] = {
+#if SSL_LIBRARY_VERSION >= 0x0920
+    { TLS1_TXT_RSA_EXPORT56_WITH_RC4_56_MD5     /*EXP56-RC4-MD5*/,    56, 128 },
+    { TLS1_TXT_RSA_EXPORT56_WITH_RC2_CBC_56_MD5 /*EXP56-RC2-CBC-MD5*/,56, 128 },
+    { TLS1_TXT_RSA_EXPORT56_WITH_DES_CBC_SHA    /*EXP56-DES-CBC-SHA*/,56,  56 },
+#endif
     { SSL3_TXT_RSA_IDEA_128_SHA          /*IDEA-CBC-SHA*/,           128, 128 },
     { SSL3_TXT_RSA_NULL_MD5              /*NULL-MD5*/,                 0,   0 },
     { SSL3_TXT_RSA_NULL_SHA              /*NULL-SHA*/,                 0,   0 },
@@ -614,7 +619,7 @@ static char *ssl_var_lookup_ssl_version(pool *p, char *var)
         result = ap_psprintf(p, "mod_ssl/%s", MOD_SSL_VERSION);
     }
     else if (strEQ(var, "LIBRARY")) {
-        result = ap_pstrdup(p, SSLeay_version(SSLEAY_VERSION));
+        result = ap_pstrdup(p, OpenSSL_version(OPENSSL_VERSION));
         if ((cp = strchr(result, ' ')) != NULL) {
             *cp = '/';
             if ((cp2 = strchr(cp, ' ')) != NULL)
